/*
 * Binary tree database internal structures, constants, etc.
 *
 * Copyright 2005 Andrew Wood, distributed under the Artistic License.
 */

#ifndef _BTREEI_H
#define _BTREEI_H 1

#define BT_RECORD_SIZE sizeof(struct bt_record)
#define BT_TOKEN_MAX   36

#ifdef __cplusplus
extern "C" {
#endif

typedef unsigned long bt_ul;

struct bt_record {              /* single binary tree record */
	bt_ul lower;               /* offset of "lower" token */
	bt_ul higher;              /* offset of "higher" token */
	unsigned char token[BT_TOKEN_MAX];   /* the token, null-terminated */
	long data[2];              /* token data (i.e. the counts) */
};

typedef struct bt_record *bt_record_t;

struct qdb_s {                  /* database state */
	int fd;                    /* file descriptor of database file */
	bt_ul size;                /* total size of database */
#ifdef HAVE_FCNTL
	int lockcount;             /* number of times lock asked for */
	int locktype;              /* type of lock to use (read or write) */
#endif
	int gotheadoffs;           /* flag, set once head offset read */
	bt_ul head_offset;         /* offset of head record of tree */
};

extern char *bt_lasterror;

#ifdef HAVE_FCNTL
int dbbt_lock(qdb_t, int);
#endif

int dbbt_read_record(qdb_t, bt_ul, bt_record_t);
int dbbt_write_record(qdb_t, bt_ul, bt_record_t);
int dbbt_find_token(qdb_t, qdb_datum, bt_record_t, bt_ul *, bt_ul *);

int dbbt_chunkread(void *, int, int, int);
int dbbt_chunkwrite(void *, int, int, int);

void dbbt_sig_block(void);
void dbbt_sig_unblock(void);

#ifdef __cplusplus
}
#endif

#endif /* _BTREEI_H */

/* EOF */
