/*
 * Locking functions for the binary tree database.
 *
 * Copyright 2005 Andrew Wood, distributed under the Artistic License.
 */

#include "config.h"
#include "database.h"

#ifdef USING_INTERNAL_DB

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <unistd.h>
#ifdef HAVE_FCNTL_H
#include <fcntl.h>
#endif
#include <sys/types.h>
#include "btreei.h"


#ifdef HAVE_FCNTL
/*
 * Obtain / release a read or write lock on the database. Returns nonzero on
 * error, and blocks until a lock can be obtained.
 */
int dbbt_lock(qdb_t db, int lock_type)
{
	struct flock lock;

	if ((lock_type == F_UNLCK) && (db->lockcount > 1)) {
		db->lockcount--;
		return 0;
	} else if ((lock_type != F_UNLCK) && (db->lockcount > 0)) {
		db->lockcount++;
		return 0;
	}

	lock.l_whence = SEEK_SET;
	lock.l_start = 0;
	lock.l_len = 0;

	lock.l_type = lock_type;

	if (fcntl(db->fd, F_SETLKW, &lock)) {
		bt_lasterror = strerror(errno);
		return 1;
	}

	if (lock_type == F_UNLCK) {
		db->lockcount--;
	} else {
		db->lockcount++;
	}

	return 0;
}
#endif				/* HAVE_FCNTL */


/*
 * Temporarily release the lock on the database.
 */
void qdb_unlock(qdb_t db)
{
#ifdef HAVE_FCNTL
	dbbt_lock(db, F_UNLCK);
#endif
}


/*
 * Reassert the lock on the database.
 */
void qdb_relock(qdb_t db)
{
#ifdef HAVE_FCNTL
	dbbt_lock(db, db->locktype);
#endif
	db->gotheadoffs = 0;
}

#endif				/* USING_INTERNAL_DB */

/* EOF */
