/*
 * Global program option structure and the parsing function prototype.
 *
 * Copyright 2005 Andrew Wood, distributed under the Artistic License.
 */

#ifndef _OPTIONS_H
#define _OPTIONS_H 1

struct opts_s;
typedef struct opts_s *opts_t;

struct opts_s {           /* structure describing run-time options */
	char *program_name;            /* name the program is running as */
	char *database;                /* location of the database file */
	char *globaldb;                /* location of global database */
	char *globaldb2;               /* location of 2nd global database */
	void *dbr1;                    /* first db handle, if any */
	void *dbr2;                    /* second db handle, if any */
	void *dbr3;                    /* third db handle, if any */
	void *dbw;                     /* db handle to write to, if any */
	void *inbuf;                   /* stdin replacement, if any */
	long inbufsize;                /* size of stdin replacement */
	unsigned char modify_subject;  /* whether to modify subject line */
	unsigned char no_header;       /* set if not to add an X-Spam line */
	unsigned char add_rating;      /* set if adding X-Spam-Rating line */
	unsigned char add_stars;       /* set if adding X-Spam-Level line */
	unsigned char no_filter;       /* set if we're not filtering */
	double threshold;              /* spam threshold (default 0.9) */
	unsigned char allowlist;       /* set if allow-list is enabled */
	unsigned char mark_spam;       /* set if marking as spam */
	unsigned char mark_nonspam;    /* set if marking as non-spam */
	unsigned int weight;           /* weighting to use when marking */
	unsigned char train;           /* set if we're training */
	unsigned char noautoprune;     /* set if we've not to auto-prune */
	unsigned char prune;           /* set if we're just pruning */
	unsigned char dump;            /* set if we're just dumping */
	unsigned char restore;         /* set if we're just restoring */
	unsigned char tokens;          /* set if we're to just dump tokens */
	unsigned char benchmark;       /* set if we're just benchmarking */
	unsigned char merge;           /* set if we're just benchmarking */
	unsigned char do_nothing;      /* set if we're not doing anything */
	unsigned int loglevel;         /* logging level (default 0) */
	unsigned int min_token_count;  /* min tokens before giving a score */
	unsigned long prune_max;       /* max tokens to prune at once */
	char *subject_marker;          /* string to add to subject if spam */
	char *mergefrom;               /* database to merge data from */
	char *emailonly;               /* email address to use in -e mode */
	int argc;                      /* number of non-option arguments */
	char **argv;                   /* array of non-option arguments */
};

extern opts_t opts_parse(int, char **);
extern void opts_free(opts_t);

#endif /* _OPTIONS_H */

/* EOF */
